package seis2viewer;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

public class Event
{

	public double lat, lon, dep, mag, rms;
	public char magtype;
	public String rawNordic;
	public Date date;
	public int year, month, day, hour, minute, nsta;
	public double second;
	final private SimpleDateFormat df = new SimpleDateFormat( "yyyyMMddHHmmss.SSS" );

	Event( String raw ) throws Exception
	{
		/* Store the raw string */
		this.rawNordic = raw.substring( 0, 80 );

		/* Lets start extracting the required strings */
		String yeaStr = raw.substring( 1, 5 ).trim();
		String monStr = raw.substring( 6, 8 ).trim();
		String dayStr = raw.substring( 8, 10 ).trim();
		String houStr = raw.substring( 11, 13 ).trim();
		String minStr = raw.substring( 13, 15 ).trim();
		String secStr = raw.substring( 16, 20 ).trim();
		String latStr = raw.substring( 23, 30 ).trim();
		String lonStr = raw.substring( 30, 38 ).trim();
		String depStr = raw.substring( 38, 43 ).trim();
		String nstStr = raw.substring( 48, 51 ).trim();
		String rmsStr = raw.substring( 51, 55 ).trim();
		String magStr[] = new String[]
		{
			raw.substring( 55, 59 ).trim(),
			raw.substring( 63, 67 ).trim(),
			raw.substring( 71, 75 ).trim()
		};
		char matStr[] = new char[]
		{
			raw.charAt( 59 ),
			raw.charAt( 67 ),
			raw.charAt( 75 )
		};

		/* Now convert extracted string to the respective formats */
		try
		{
			if( yeaStr.isEmpty() )
				year = 0;
			else
				year = Integer.valueOf( yeaStr );
		}
		catch( Exception e )
		{
			throw new Exception( "Error parsing year" );
		}
		try
		{
			if( monStr.isEmpty() )
				month = 1;
			else
				month = Integer.valueOf( monStr );
		}
		catch( Exception e )
		{
			throw new Exception( "Error parsing month" );
		}
		try
		{
			if( dayStr.isEmpty() )
				day = 1;
			else
				day = Integer.valueOf( dayStr );
		}
		catch( Exception e )
		{
			throw new Exception( "Error parsing day" );
		}
		try
		{
			if( houStr.isEmpty() )
				hour = 0;
			else
				hour = Integer.valueOf( houStr );
		}
		catch( Exception e )
		{
			throw new Exception( "Error parsing hour" );
		}
		try
		{
			if( minStr.isEmpty() )
				minute = 0;
			else
				minute = Integer.valueOf( minStr );
		}
		catch( Exception e )
		{
			throw new Exception( "Error parsing minute" );
		}
		try
		{
			if( secStr.isEmpty() )
				second = 0;
			else
				second = Double.valueOf( secStr );
		}
		catch( Exception e )
		{
			throw new Exception( "Error parsing seconds" );
		}
		try
		{
			String dateStr = String.format( Locale.UK, "%04d%02d%02d%02d%02d%06.3f",
					year, month, day, hour, minute, second );

			this.date = df.parse( dateStr );
		}
		catch( Exception e )
		{
			throw new Exception( "Unable to parse date" );
		}

		try
		{
			if( latStr.isEmpty() )
				lat = 0;
			else
				this.lat = Double.valueOf( latStr );
		}
		catch( Exception e )
		{
			throw new Exception( "Unable to parse latitude <" + latStr + ">" );
		}

		try
		{
			if( lonStr.isEmpty() )
				lon = 0;
			else
				this.lon = Double.valueOf( lonStr );
		}
		catch( Exception e )
		{
			throw new Exception( "Unable to parse longitude <" + lonStr + ">" );
		}
		try
		{
			if( depStr.isEmpty() )
				dep = 0;
			else
				this.dep = Double.valueOf( depStr );
		}
		catch( Exception e )
		{
			throw new Exception( "Unable to parse depth <" + depStr + ">" );
		}
		try
		{
			if( nstStr.isEmpty() )
				nsta = 0;
			else
				this.nsta = Integer.valueOf( nstStr );
		}
		catch( Exception e )
		{
			throw new Exception( "Unable to parse number of stations <" + nstStr + ">" );
		}
		try
		{
			if( rmsStr.isEmpty() )
				rms = 0;
			else
				this.rms = Double.valueOf( rmsStr );
		}
		catch( Exception e )
		{
			throw new Exception( "Unable to parse RMS: <" + rmsStr + ">" );
		}
		String fmagStr = null;
		for( int i = 0; i < 3; i++ )
			if( !magStr[i].isEmpty() )
			{
				fmagStr = magStr[i];
				this.magtype = matStr[i];
				break;
			}
		if( fmagStr == null )
		{
			//System.err.println( "Error with <" + raw + ">: Unable to find magnitudes" );
		}
		else
		{
			mag = -9.9;
			try
			{
				this.mag = Double.valueOf( fmagStr );
			}
			catch( Exception e )
			{
				throw new Exception( "Unable to parse magnitude" );
				
			}
		}
	}

	public String toNLL()
	{
		String out = String.format( Locale.UK, "NLLOC \"%s\"\n"
				+ "SIGNATURE \"seis2viewer\"\nCOMMENT \"Automatic Conversion\"\n"
				+ "GEOGRAPHIC  OT %04d %02d %02d  %02d %02d %09.6f  Lat  %9.6f Long %11.6f Depth %11.6f\n"
				+ "QUALITY  Pmax 1.0 MFmin 0.5 MFmax 6.0 RMS %8.6f Nphs %d Gap 90 Dist 10.0 Mamp  %.2f 0 Mdur -9.90 0\n"
				+ "TRANSFORM SIMPLE latOrig %4.1f longOrig %5.1f rotAngle 0.0\n"
				+ "END_NLLOC",
				rawNordic, year, month, day, hour, minute, second, lat, lon, dep,
				rms, nsta, mag, lat, lon );
		return out;
	}
}
